// CoreCalc, a spreadsheet core implementation 

// ----------------------------------------------------------------------
// Copyright (c) 2006 Thomas S. Iversen

// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

//  * The above copyright notice and this permission notice shall be
//    included in all copies or substantial portions of the Software.

//  * The software is provided "as is", without warranty of any kind,
//    express or implied, including but not limited to the warranties of
//    merchantability, fitness for a particular purpose and
//    noninfringement.  In no event shall the authors or copyright
//    holders be liable for any claim, damages or other liability,
//    whether in an action of contract, tort or otherwise, arising from,
//    out of or in connection with the software or the use or other
//    dealings in the software.
// ----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Diagnostics; // Stopwatch
using CoreCalc;

// Example benchmark: Evaluate many occurrences of SIN($A$1)

namespace Benchmarks {
  static class BenchmarkExample {
    static void Main(String[] args) {
      String filename = null;
      if (args != null && args.Length > 0) {
        if (args.Length > 1) {
          Console.WriteLine("Usage: BenchmarkExample <xmlfilename>");
          return;
        } else {
          filename = args[0];
        }
      }

      Stopwatch watch = new Stopwatch();

      String title = "Simple Math function, reference argument (SIN($A$1))";
      String author = "Thomas S. Iversen";

      // Number of Instances: 8192 * 32 = 262144
      int cols = 32; int rows = 8192;
      int startcol = 1;   // Column B
      int runs = 3;

      // Set up workbook, sheet, formulas
      Workbook wb = new Workbook();
      Sheet sheet = new Sheet(wb, "TestSheet", cols + 1, rows);

      sheet.InsertCell("=SIN($A$1)", new CellAddr("B1"));
      Cell cell = sheet[new CellAddr("B1")];

      sheet.InsertCell("=PI()/4", new CellAddr("A1"));
      for (int col = startcol; col < cols + startcol; col++) {
        for (int row = 0; row < rows; row++) {
          if (!(col == startcol && row == 0))
            sheet.PasteCell(cell, new CellAddr(col, row));
        }
      }

      // Run benchmark
      Benchmark benchmark = new Benchmark(title, author);
      XmlElement dataset = benchmark.AddDataSet("Evaluation time", "ms");
      XmlElement data = benchmark.AddData(dataset, "CoreCalc Eval");
      XmlElement subdata1 = benchmark.AddSubData(data, "Evaluation time", "ms", runs);
      for (int run = 1; run <= runs; run++) {
        watch.Reset();
        watch.Start();
        wb.Recalculate();
        watch.Stop();
        benchmark.AddRun(subdata1, run, watch.ElapsedMilliseconds.ToString());
      }
      Console.WriteLine("Value: {0}", sheet.ShowValue(startcol, 0));

      if (filename != null) 
        benchmark.Save(filename);
    }
  }
}