// CoreCalc, a spreadsheet core implementation 

// ----------------------------------------------------------------------
// Copyright (c) 2006 Peter Sestoft

// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

//  * The above copyright notice and this permission notice shall be
//    included in all copies or substantial portions of the Software.

//  * The software is provided "as is", without warranty of any kind,
//    express or implied, including but not limited to the warranties of
//    merchantability, fitness for a particular purpose and
//    noninfringement.  In no event shall the authors or copyright
//    holders be liable for any claim, damages or other liability,
//    whether in an action of contract, tort or otherwise, arising from,
//    out of or in connection with the software or the use or other
//    dealings in the software.
// ----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Diagnostics; // Stopwatch
using CoreCalc;

// Benchmark of two implementations of SUM(...)

namespace Benchmarks {
  static class SumFunctions {
    static void Main(String[] args) {
      String filename = "SumFunctions.xml";
      Stopwatch watch = new Stopwatch();

      String title = "Fastsum function benchmark";
      String author = "Peter Sestoft";

      Benchmark benchmark = new Benchmark(title, author);
      XmlElement dataset = benchmark.AddDataSet("Evaluation time", "ms");

      // Number of Instances: 8192 * 32 = 262144
      int cols = 2; int rows = 12288;
      int runs = 3;

      // Benchmark FASTSUM function
      {
        Workbook wb = new Workbook();
        Sheet sheet = new Sheet(wb, "TestSheet", cols + 1, rows);

        sheet.InsertCell("0.5", new CellAddr("A1"));
        Cell a2Cell = sheet.InsertCell("=A1*1.00001", new CellAddr("A2"));
        sheet.PasteCell(a2Cell, new CellAddr("A3"), 1, rows - 2);

        Cell b1Cell = sheet.InsertCell("=FASTSUM(A1:A$1)", new CellAddr("B1"));
        sheet.PasteCell(b1Cell, new CellAddr("B2"), 1, rows - 1);

        XmlElement data = benchmark.AddData(dataset, "FASTSUM eval");
        DoRecalculation(watch, benchmark, runs, wb, data);
        Console.WriteLine("Value: {0}", sheet.ShowValue(1, rows - 1));
      }

      // Benchmark ordinary SUM function

      {
        Workbook wb = new Workbook();
        Sheet sheet = new Sheet(wb, "TestSheet", cols + 1, rows);

        sheet.InsertCell("0.5", new CellAddr("A1"));
        Cell a2Cell = sheet.InsertCell("=A1*1.00001", new CellAddr("A2"));
        sheet.PasteCell(a2Cell, new CellAddr("A3"), 1, rows - 2);

        Cell b1Cell = sheet.InsertCell("=SUM(A1:A$1)", new CellAddr("B1"));
        sheet.PasteCell(b1Cell, new CellAddr("B2"), 1, rows - 1);

        XmlElement data = benchmark.AddData(dataset, "SUM eval");
        DoRecalculation(watch, benchmark, runs, wb, data);
        Console.WriteLine("Value: {0}", sheet.ShowValue(1, rows - 1));
      }

      if (filename != null)
        benchmark.Save(filename);
    }

    private static void DoRecalculation(Stopwatch watch, Benchmark benchmark, int runs, Workbook wb, XmlElement data) {
      XmlElement subdata1 = benchmark.AddSubData(data, "Evaluation time", "ms", runs);
      for (int run = 1; run <= runs; run++) {
        watch.Reset();
        watch.Start();
        wb.Recalculate();
        watch.Stop();
        benchmark.AddRun(subdata1, run, watch.ElapsedMilliseconds.ToString());
      }

    }
  }
}