using System.Collections;

using System;

namespace CoreCalc {



public class Parser {
	const int _EOF = 0;
	const int _name = 1;
	const int _number = 2;
	const int _sheetref = 3;
	const int _raref = 4;
	const int _xmlssraref11 = 5;
	const int _xmlssraref12 = 6;
	const int _xmlssraref13 = 7;
	const int _xmlssraref21 = 8;
	const int _xmlssraref22 = 9;
	const int _xmlssraref23 = 10;
	const int _xmlssraref31 = 11;
	const int _xmlssraref32 = 12;
	const int _xmlssraref33 = 13;
	const int _string = 14;
	const int _textcell = 15;
	const int maxT = 34;

	const bool T = true;
	const bool x = false;
	const int minErrDist = 2;
	
	public Scanner scanner;
	public Errors  errors;

	public Token t;    // last recognized token
	public Token la;   // lookahead token
	int errDist = minErrDist;

private int col, row;
  private Workbook workbook;
  private Cell cell;
  System.Globalization.NumberFormatInfo ni = null;
  
  public void SetNumberDecimalSeparator(String nds) {
         System.Globalization.CultureInfo ci = 
            System.Globalization.CultureInfo.InstalledUICulture;
         this.ni = (System.Globalization.NumberFormatInfo)
            ci.NumberFormat.Clone();
         this.ni.NumberDecimalSeparator = nds;
  }


  public Cell ParseCell(Workbook workbook, int col, int row) {
    this.workbook = workbook;
    this.col = col; this.row = row;
    
    // US decimal point, regardless of culture
    SetNumberDecimalSeparator(".");
    Parse();
    return cell;
  }

/*--------------------------------------------------------------------------*/


	public Parser(Scanner scanner) {
		this.scanner = scanner;
		errors = new Errors();
	}

	void SynErr (int n) {
		if (errDist >= minErrDist) errors.SynErr(la.line, la.col, n);
		errDist = 0;
	}

	public void SemErr (string msg) {
		if (errDist >= minErrDist) errors.Error(t.line, t.col, msg);
		errDist = 0;
	}
	
	void Get () {
		for (;;) {
			t = la;
			la = scanner.Scan();
			if (la.kind <= maxT) { ++errDist; break; }

			la = t;
		}
	}
	
	void Expect (int n) {
		if (la.kind==n) Get(); else { SynErr(n); }
	}
	
	bool StartOf (int s) {
		return set[s, la.kind];
	}
	
	void ExpectWeak (int n, int follow) {
		if (la.kind == n) Get();
		else {
			SynErr(n);
			while (!StartOf(follow)) Get();
		}
	}
	
	bool WeakSeparator (int n, int syFol, int repFol) {
		bool[] s = new bool[maxT+1];
		if (la.kind == n) { Get(); return true; }
		else if (StartOf(repFol)) return false;
		else {
			for (int i=0; i <= maxT; i++) {
				s[i] = set[syFol, i] || set[repFol, i] || set[0, i];
			}
			SynErr(n);
			while (!s[la.kind]) Get();
			return StartOf(syFol);
		}
	}
	
	void AddOp(out String op) {
		op = "+"; 
		if (la.kind == 16) {
			Get();
		} else if (la.kind == 17) {
			Get();
			op = "-"; 
		} else if (la.kind == 18) {
			Get();
			op = "&"; 
		} else SynErr(35);
	}

	void LogicalOp(out String op) {
		op = "=="; 
		switch (la.kind) {
		case 19: {
			Get();
			break;
		}
		case 20: {
			Get();
			op = "<>"; 
			break;
		}
		case 21: {
			Get();
			op = "<";  
			break;
		}
		case 22: {
			Get();
			op = "<="; 
			break;
		}
		case 23: {
			Get();
			op = ">";  
			break;
		}
		case 24: {
			Get();
			op = ">="; 
			break;
		}
		default: SynErr(36); break;
		}
	}

	void Expr(out Expr e) {
		Expr e2; String op; e = null; 
		LogicalTerm(out e);
		while (StartOf(1)) {
			LogicalOp(out op);
			LogicalTerm(out e2);
			e = new FunCall(op, new Expr[] { e, e2 }); 
		}
	}

	void LogicalTerm(out Expr e) {
		Expr e2; String op; e = null; 
		Term(out e);
		while (la.kind == 16 || la.kind == 17 || la.kind == 18) {
			AddOp(out op);
			Term(out e2);
			e = new FunCall(op, new Expr[] { e, e2 }); 
		}
	}

	void Term(out Expr e) {
		Expr e2; String op; 
		PowFactor(out e);
		while (la.kind == 31 || la.kind == 32) {
			MulOp(out op);
			PowFactor(out e2);
			e = new FunCall(op, new Expr[] { e, e2 } ); 
		}
	}

	void Factor(out Expr e) {
		RARef r1, r2; Sheet s1 = null; double d; e = null; 
		if (StartOf(2)) {
			if (StartOf(3)) {
			} else {
				Get();
				String sheetname = t.val.TrimEnd("!".ToCharArray());
				s1 = workbook[sheetname];
				
			}
			Raref(out r1);
			if (StartOf(4)) {
				e = new CellRef(s1, r1);      
			} else if (la.kind == 25) {
				Get();
				Raref(out r2);
				e = new CellArea(null, r1, r2); 
			} else SynErr(37);
		} else if (la.kind == 2 || la.kind == 17) {
			Number(out d);
			e = new NumberConst(d);         
		} else if (la.kind == 14) {
			Get();
			int len = t.val.Length-2;
			e = new TextConst(t.val.Substring(1, len)); 
			
		} else if (la.kind == 26) {
			Get();
			Expr(out e);
			Expect(27);
		} else if (la.kind == 1) {
			Application(out e);
		} else SynErr(38);
	}

	void Raref(out RARef raref) {
		raref = null;
		switch (la.kind) {
		case 4: {
			Get();
			raref = new RARef(t.val, col, row); 
			break;
		}
		case 5: {
			Get();
			raref = new RARef(t.val); 
			break;
		}
		case 6: {
			Get();
			raref = new RARef(t.val); 
			break;
		}
		case 7: {
			Get();
			raref = new RARef(t.val); 
			break;
		}
		case 8: {
			Get();
			raref = new RARef(t.val); 
			break;
		}
		case 9: {
			Get();
			raref = new RARef(t.val); 
			break;
		}
		case 10: {
			Get();
			raref = new RARef(t.val); 
			break;
		}
		case 11: {
			Get();
			raref = new RARef(t.val); 
			break;
		}
		case 12: {
			Get();
			raref = new RARef(t.val); 
			break;
		}
		case 13: {
			Get();
			raref = new RARef(t.val); 
			break;
		}
		default: SynErr(39); break;
		}
	}

	void Number(out double d) {
		d = 0.0;                      
		if (la.kind == 2) {
			Get();
			d = double.Parse(t.val, ni); 
		} else if (la.kind == 17) {
			Get();
			Expect(2);
			d = -double.Parse(t.val, ni); 
		} else SynErr(40);
	}

	void Application(out Expr e) {
		String s; Expr[] es; e = null; 
		Name(out s);
		Expect(26);
		if (la.kind == 27) {
			Get();
			e = new FunCall(s, new Expr[0]); 
		} else if (StartOf(5)) {
			Exprs1(out es);
			Expect(27);
			e = new FunCall(s, es); 
		} else SynErr(41);
	}

	void PowFactor(out Expr e) {
		Expr e2; 
		Factor(out e);
		while (la.kind == 28) {
			Get();
			Factor(out e2);
			e = new FunCall("^", new Expr[] { e, e2 } ); 
		}
	}

	void Name(out String s) {
		Expect(1);
		s = t.val; 
	}

	void Exprs1(out Expr[] es) {
		Expr e1, e2; 
		ArrayList elist = new ArrayList(); 
		
		Expr(out e1);
		elist.Add(e1); 
		while (la.kind == 29 || la.kind == 30) {
			if (la.kind == 29) {
				Get();
			} else {
				Get();
			}
			Expr(out e2);
			elist.Add(e2); 
		}
		es = new Expr[elist.Count];
		elist.CopyTo(es, 0); 
		
	}

	void MulOp(out String op) {
		op = "*"; 
		if (la.kind == 31) {
			Get();
		} else if (la.kind == 32) {
			Get();
			op = "/"; 
		} else SynErr(42);
	}

	void Spreadsheet() {
		Expr e; double d;     
		if (la.kind == 33) {
			Get();
			Expr(out e);
			this.cell = new Formula(workbook, e); 
		} else if (la.kind == 15) {
			Get();
			this.cell = new TextCell(t.val.Substring(1)); 
		} else if (la.kind == 2 || la.kind == 17) {
			Number(out d);
			this.cell = new NumberCell(d);  
		} else if (la.kind == 14) {
			Get();
			this.cell = new TextCell(t.val.Substring(1)); 
		} else SynErr(43);
	}



	public void Parse() {
		la = new Token();
		la.val = "";		
		Get();
		Spreadsheet();

    Expect(0);
	}
	
	bool[,] set = {
		{T,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x},
		{x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,T, T,T,T,T, T,x,x,x, x,x,x,x, x,x,x,x},
		{x,x,x,T, T,T,T,T, T,T,T,T, T,T,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x},
		{x,x,x,x, T,T,T,T, T,T,T,T, T,T,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x},
		{T,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, T,T,T,T, T,T,T,T, T,x,x,T, T,T,T,T, T,x,x,x},
		{x,T,T,T, T,T,T,T, T,T,T,T, T,T,T,x, x,T,x,x, x,x,x,x, x,x,T,x, x,x,x,x, x,x,x,x}

	};
} // end Parser


public class Errors {
	public int count = 0;                                    // number of errors detected
  public string errMsgFormat = "-- line {0} col {1}: {2}"; // 0=line, 1=column, 2=text
	
	public void SynErr (int line, int col, int n) {
		string s;
		switch (n) {
			case 0: s = "EOF expected"; break;
			case 1: s = "name expected"; break;
			case 2: s = "number expected"; break;
			case 3: s = "sheetref expected"; break;
			case 4: s = "raref expected"; break;
			case 5: s = "xmlssraref11 expected"; break;
			case 6: s = "xmlssraref12 expected"; break;
			case 7: s = "xmlssraref13 expected"; break;
			case 8: s = "xmlssraref21 expected"; break;
			case 9: s = "xmlssraref22 expected"; break;
			case 10: s = "xmlssraref23 expected"; break;
			case 11: s = "xmlssraref31 expected"; break;
			case 12: s = "xmlssraref32 expected"; break;
			case 13: s = "xmlssraref33 expected"; break;
			case 14: s = "string expected"; break;
			case 15: s = "textcell expected"; break;
			case 16: s = "\"+\" expected"; break;
			case 17: s = "\"-\" expected"; break;
			case 18: s = "\"&\" expected"; break;
			case 19: s = "\"==\" expected"; break;
			case 20: s = "\"<>\" expected"; break;
			case 21: s = "\"<\" expected"; break;
			case 22: s = "\"<=\" expected"; break;
			case 23: s = "\">\" expected"; break;
			case 24: s = "\">=\" expected"; break;
			case 25: s = "\":\" expected"; break;
			case 26: s = "\"(\" expected"; break;
			case 27: s = "\")\" expected"; break;
			case 28: s = "\"^\" expected"; break;
			case 29: s = "\";\" expected"; break;
			case 30: s = "\",\" expected"; break;
			case 31: s = "\"*\" expected"; break;
			case 32: s = "\"/\" expected"; break;
			case 33: s = "\"=\" expected"; break;
			case 34: s = "??? expected"; break;
			case 35: s = "invalid AddOp"; break;
			case 36: s = "invalid LogicalOp"; break;
			case 37: s = "invalid Factor"; break;
			case 38: s = "invalid Factor"; break;
			case 39: s = "invalid Raref"; break;
			case 40: s = "invalid Number"; break;
			case 41: s = "invalid Application"; break;
			case 42: s = "invalid MulOp"; break;
			case 43: s = "invalid Spreadsheet"; break;

			default: s = "error " + n; break;
		}
		Console.WriteLine(errMsgFormat, line, col, s);
		count++;
	}

	public void SemErr (int line, int col, int n) {
		Console.WriteLine(errMsgFormat, line, col, ("error " + n));
		count++;
	}

	public void Error (int line, int col, string s) {
		Console.WriteLine(errMsgFormat, line, col, s);
		count++;
	}

	public void Exception (string s) {
		Console.WriteLine(s); 
		System.Environment.Exit(1);
	}
} // Errors

}